<?php
    require "vendor/autoload.php";

    class OxaPay extends PaymentGatewayModule
    {
        function __construct()
        {
            $this->name = __CLASS__;

            parent::__construct();
        }

        public function config_fields()
        {
            return [
                'merchant' => [
                    'name'              => $this->lang['name_merchant'],
                    'description'       => $this->lang['description_merchant'],
                    'type'              => "text",
                    'value'             => $this->config['settings']['merchant'] ?? '',
                    'placeholder'       => $this->lang['name_merchant'],
                ],
                'lifetime'          => [
                    'name'              => $this->lang['name_lifetime'],
                    'description'       => $this->lang['description_lifetime'],
                    'type'              => "text",
                    'value'             => $this->config['settings']['lifetime'] ?? '60',
                ],
            ];
        }

        public function area($params=[])
        {
            $url = 'https://api.oxapay.com/merchants/request';
            $order_id = $this->checkout_id;
            $merchant = $this->config["settings"]["merchant"] ?? "";
            $lifetime = $this->config['settings']['lifetime'] ?? 60;
            $data = array(
                'merchant' => $merchant,
                'amount' => (string)$params['amount'],
                'currency' => $params['currency'],
                'orderId' => $order_id,
                'callbackUrl' => $this->links['callback'],
                'returnUrl' => $this->checkout['data']['redirect']['success'],
                'lifeTime' => (string)((int)$lifetime),
            );
            $options = array(
                'http' => array(
                    'header' => 'Content-Type: application/json',
                    'method'  => 'POST',
                    'content' => json_encode($data),
                ),
            );
            $context  = stream_context_create($options);
            $response = file_get_contents($url, false, $context);
            $result = json_decode($response);
            $invoice_link = $result->payLink;
			if ($result->result !== 100 ) {
				if (!empty($result->message)) {
					echo 'Error OxaPay create payment. Message: ' . $result->message;
					exit;
				}
	
			}
            return
                '<a href="'.$invoice_link.'" target="_blank">'.$this->lang["pay-button"].'</a>';
        }

        public function callback()
        {
            $postData = file_get_contents('php://input');
			$data = json_decode($postData);
            if ($data->type === 'payment') {
                $apiSecretKey = $this->config["settings"]["merchant"];
            } 
            else {
                http_response_code(400);
                echo 'Invalid data.type';
                exit;
            }
            $hmacHeader = $_SERVER['HTTP_HMAC'];
            $calculatedHmac = hash_hmac('sha512', $postData, $apiSecretKey);
            if ($calculatedHmac === $hmacHeader) {
                $order_id = $data->orderId;
                if(!$order_id){
                    $this->error = 'Order id is not found.';
                    return false;
                }
                $checkout = $this->get_checkout($order_id);
    
                if(!$checkout)
                {
                    $this->error = 'Checkout ID unknown';
                    return false;
                }
    
                $status = $this->convertToWisecpStatus($data->status);
                $this->set_checkout($checkout);
    
                $message = [
                    'Merchant Transaction ID' => $order_id,
                ];
    
                $response = [
                    'message' => $message,
                    'callback_message' => 200,
                ];
            
                if ($status === 'successful') {
                    $response['status'] = 'successful';
                    return $response;
                }
                elseif($status === 'error'){
                    $response['status'] = 'error';
                    return $response;
                }
                else {
                    return 200;
                }

            }
            else {
                http_response_code(400);
                echo 'Invalid HMAC signature';
            }
        }

        private function convertToWisecpStatus($status) {
            $result = 'pending';
            switch($status) {
                case 'Confirming': 
                case 'Paid':
                    $result = 'successful';
                    break;
                case 'Waiting':
                    $result = 'pending';
                    break;

                case 'Expired':
                case 'Failed': 
                    $result = 'error';
                    break;
            }
            return $result;
        }

    }
